@@ -108,7 +108,7 @@ class DTestBase(object):
     _class_attributes = [
         '_name', '_test', '_class', '_exp_fail', '_skip', '_pre', '_post',
         '_deps', '_revdeps', '_partner', '_attrs', '_raises', '_timeout',
-        '_result', '_repeat', '_strategy', '_policy'
+        '_result', '_repeat', '_strategy', '_policy', '_resources'
         ]
 
     def __init__(self, test):
@@ -161,6 +161,7 @@ class DTestBase(object):
         self._repeat = 1
         self._strategy = strat.SerialStrategy()
         self._policy = pol.basicPolicy
+        self._resources = {}
 
         # Attach ourself to the test
         test._dt_dtest = self
@@ -404,6 +405,18 @@ class DTestBase(object):
         # like an attribute
         return self._repeat
 
+    @property
+    def resources(self):
+        """
+        Retrieve the resources for this test.  Returns a shallow copy
+        of the resources dictionary, as set up by the @require()
+        decorator.
+        """
+
+        # We want the resources to be read-only, but to be accessed
+        # like an attribute
+        return self._resources.copy()
+
     @classmethod
     def promote(cls, test):
         """
@@ -498,7 +511,7 @@ class DTestBase(object):
         # Re-set our name
         self._name = None
 
-    def _run(self, output):
+    def _run(self, output, res_mgr):
         """
         Perform the test.  Causes any fixtures discovered as part of
         the class or explicitly set (or overridden) by the setUp() and
@@ -533,6 +546,15 @@ class DTestBase(object):
             if not self._result:
                 pre_status = False
 
+        # Get the test resources...
+        resgen = None
+        if pre_status:
+            with self._result.accumulate(PRE):
+                resgen = res_mgr.collect(self._resources)
+                resources = resgen.next()
+            if not self._result:
+                pre_status = False
+
         # Execute the test
         if pre_status:
             # Prepare the strategy...
@@ -540,7 +562,7 @@ class DTestBase(object):
 
             # Trigger the test
             self._trigger(self._test.__name__,
-                          get_call(self._test, obj), (), {})
+                          get_call(self._test, obj), (), resources)
 
             # Wait for spawned threads
             self._strategy.wait()
@@ -554,6 +576,16 @@ class DTestBase(object):
         # Transition to the appropriate ending state
         self._result._transition(output=output)
 
+        # Clean up the resources
+        if resgen:
+            try:
+                # We don't allow this to fail; failures in tearDown()
+                # methods are pooled together and printed after all
+                # the test failures
+                resgen.send(str(self.result))
+            except StopIteration:
+                pass
+
         # Return the result
         return self._result
 
@@ -1280,6 +1312,30 @@ def threshold(th):
     return wrapper
 
 
+def require(**resources):
+    """
+    Decorates a test to indicate that the test requires certain
+    resources.  Resources are specified as keyword arguments to the
+    decorator, with the values being instances of subclasses of
+    Resource; allocated versions of those resources will be passed in
+    corresponding keyword arguments to the test function.
+    """
+
+    # Need a wrapper to perform the actual decoration
+    def wrapper(func):
+        # Get the DTest object for the test
+        dt = _gettest(func)
+
+        # Store the required resources
+        dt._resources.update(resources)
+
+        # Return the function
+        return func
+
+    # Return the actual decorator
+    return wrapper
+
+
 testRE = re.compile(r'(?:^|[\b_\.-])[Tt]est')
 
 
